@@ -30,7 +30,11 @@ makeEditText pm df AddMinimalMethodsParams{..} = do
                 pm_parsed_source pm
 
         old = T.pack $ exactPrint ps
+#if MIN_VERSION_ghc_exactprint(1,10,0)
+        ps' = addMethodDecls ps mDecls range withSig
+#else
         (ps', _, _) = runTransform (addMethodDecls ps mDecls range withSig)
+#endif
         new = T.pack $ exactPrint ps'
     pure (old, new)
 
@@ -40,7 +44,9 @@ makeMethodDecl df (mName, sig) = do
     sig' <- eitherToMaybe $ parseDecl df (T.unpack sig) $ T.unpack sig
     pure (name, sig')
 
-#if MIN_VERSION_ghc(9,5,0)
+#if MIN_VERSION_ghc_exactprint(1,10,0)
+addMethodDecls :: ParsedSource -> [(LHsDecl GhcPs, LHsDecl GhcPs)] -> Range -> Bool -> Located (HsModule GhcPs)
+#elif MIN_VERSION_ghc(9,5,0)
 addMethodDecls :: ParsedSource -> [(LHsDecl GhcPs, LHsDecl GhcPs)] -> Range -> Bool -> TransformT Identity (Located (HsModule GhcPs))
 #else
 addMethodDecls :: ParsedSource -> [(LHsDecl GhcPs, LHsDecl GhcPs)] -> Range -> Bool -> TransformT Identity (Located HsModule)
@@ -50,12 +56,20 @@ addMethodDecls ps mDecls range withSig
     | otherwise = go (map fst mDecls)
     where
     go inserting = do
+#if MIN_VERSION_ghc_exactprint(1,10,0)
+        let allDecls = hsDecls ps
+#else
         allDecls <- hsDecls ps
+#endif
         case break (inRange range . getLoc) allDecls of
             (before, L l inst : after) ->
                 let
                     instSpan = realSrcSpan $ getLoc l
+#if MIN_VERSION_ghc(9,11,0)
+                    instCol = srcSpanStartCol instSpan - 1
+#else
                     instCol = srcSpanStartCol instSpan
+#endif
 #if MIN_VERSION_ghc(9,9,0)
                     instRow = srcSpanEndLine instSpan
                     methodEpAnn = noAnnSrcSpanDP $ deltaPos 1 (instCol + defaultIndent)
@@ -91,7 +105,17 @@ addMethodDecls ps mDecls range withSig
     addWhere :: HsDecl GhcPs -> HsDecl GhcPs
     addWhere instd@(InstD xInstD (ClsInstD ext decl@ClsInstDecl{..})) =
         case cid_ext of
-#if MIN_VERSION_ghc(9,9,0)
+#if MIN_VERSION_ghc(9,11,0)
+            (warnings, anns, key)
+                | EpTok _ <- acid_where anns -> instd
+                | otherwise ->
+                    InstD xInstD (ClsInstD ext decl {
+                    cid_ext = ( warnings
+                              , anns { acid_where = EpTok d1 }
+                              , key
+                              )
+                    })
+#elif MIN_VERSION_ghc(9,9,0)
             (warnings, anns, key)
                 | any (\(AddEpAnn kw _ )-> kw == AnnWhere) anns -> instd
                 | otherwise ->
