@@ -5,7 +5,7 @@
 {-# LANGUAGE OverloadedStrings         #-}
 {-# LANGUAGE RecordWildCards           #-}
 {-# LANGUAGE ViewPatterns              #-}
-{-# OPTIONS_GHC -fno-warn-type-defaults -Wno-unused-imports #-}
+{-# OPTIONS_GHC -Wno-type-defaults #-}
 
 {- |
 A plugin inspired by the REPLoid feature of <https://github.com/jyp/dante Dante>, <https://www.haskell.org/haddock/doc/html/ch03s08.html#idm140354810775744 Haddock>'s Examples and Properties and <https://hackage.haskell.org/package/doctest Doctest>.
@@ -18,13 +18,12 @@ module Ide.Plugin.Eval.CodeLens (
 ) where
 
 import           Control.Applicative                          (Alternative ((<|>)))
-import           Control.Arrow                                (second, (>>>))
-import           Control.Exception                            (bracket_, try)
+import           Control.Arrow                                (second)
+import           Control.Exception                            (bracket_)
 import qualified Control.Exception                            as E
-import           Control.Lens                                 (_1, _3, ix, (%~),
-                                                               (<&>), (^.))
-import           Control.Monad                                (guard, join,
-                                                               void, when)
+import           Control.Lens                                 (ix, (%~), (^.))
+import           Control.Monad                                (guard, void,
+                                                               when)
 import           Control.Monad.IO.Class                       (MonadIO (liftIO))
 import           Control.Monad.Trans.Except                   (ExceptT (..),
                                                                runExceptT)
@@ -40,29 +39,23 @@ import           Data.Maybe                                   (catMaybes)
 import           Data.String                                  (IsString)
 import           Data.Text                                    (Text)
 import qualified Data.Text                                    as T
-import           Data.Typeable                                (Typeable)
+import qualified Data.Text.Utf16.Rope.Mixed                   as Rope
+import           Development.IDE.Core.FileStore               (getUriContents)
 import           Development.IDE.Core.Rules                   (IdeState,
                                                                runAction)
 import           Development.IDE.Core.RuleTypes               (LinkableResult (linkableHomeMod),
-                                                               NeedsCompilation (NeedsCompilation),
                                                                TypeCheck (..),
                                                                tmrTypechecked)
-import           Development.IDE.Core.Shake                   (shakeExtras,
-                                                               useNoFile_,
-                                                               useWithStale_,
-                                                               use_, uses_)
+import           Development.IDE.Core.Shake                   (useNoFile_, use_,
+                                                               uses_)
 import           Development.IDE.GHC.Compat                   hiding (typeKind,
                                                                unitState)
-import           Development.IDE.GHC.Compat.Util              (GhcException,
-                                                               OverridingBool (..),
-                                                               bagToList)
+import           Development.IDE.GHC.Compat.Util              (OverridingBool (..))
 import           Development.IDE.GHC.Util                     (evalGhcEnv,
-                                                               modifyDynFlags,
-                                                               printOutputable)
+                                                               modifyDynFlags)
 import           Development.IDE.Import.DependencyInformation (transitiveDeps,
                                                                transitiveModuleDeps)
-import           Development.IDE.Types.Location               (toNormalizedFilePath',
-                                                               uriToFilePath')
+import           Development.IDE.Types.Location               (toNormalizedFilePath')
 import           GHC                                          (ClsInst,
                                                                ExecOptions (execLineNumber, execSourceFile),
                                                                FamInst,
@@ -87,15 +80,12 @@ import           Development.IDE.Core.RuleTypes               (GetLinkable (GetL
                                                                ModSummaryResult (msrModSummary))
 import           Development.IDE.Core.Shake                   (VFSModified (VFSUnmodified))
 import qualified Development.IDE.GHC.Compat.Core              as Compat (InteractiveImport (IIModule))
-import qualified Development.IDE.GHC.Compat.Core              as SrcLoc (HasSrcSpan (getLoc),
-                                                                         unLoc)
+import qualified Development.IDE.GHC.Compat.Core              as SrcLoc (unLoc)
 import           Development.IDE.Types.HscEnvEq               (HscEnvEq (hscEnv))
 import qualified GHC.LanguageExtensions.Type                  as LangExt (Extension (..))
 
-import           Control.Concurrent.STM.Stats                 (atomically)
 import           Development.IDE.Core.FileStore               (setSomethingModified)
 import           Development.IDE.Core.PluginUtils
-import           Development.IDE.Graph                        (ShakeOptions (shakeExtra))
 import           Development.IDE.Types.Shake                  (toKey)
 import           GHC.Types.SrcLoc                             (UnhelpfulSpanReason (UnhelpfulInteractive))
 import           Ide.Logger                                   (Priority (..),
@@ -103,7 +93,6 @@ import           Ide.Logger                                   (Priority (..),
                                                                WithPriority,
                                                                logWith)
 import           Ide.Plugin.Error                             (PluginError (PluginInternalError),
-                                                               handleMaybe,
                                                                handleMaybeM)
 import           Ide.Plugin.Eval.Code                         (Statement,
                                                                asStatements,
@@ -117,8 +106,7 @@ import           Ide.Plugin.Eval.Config                       (EvalConfig (..),
 import           Ide.Plugin.Eval.GHC                          (addImport,
                                                                addPackages,
                                                                hasPackage,
-                                                               setSessionAndInteractiveDynFlags,
-                                                               showDynFlags)
+                                                               setSessionAndInteractiveDynFlags)
 import           Ide.Plugin.Eval.Parse.Comments               (commentsToSections)
 import           Ide.Plugin.Eval.Parse.Option                 (parseSetFlags)
 import           Ide.Plugin.Eval.Rules                        (queueForEvaluation,
@@ -133,7 +121,10 @@ import qualified Language.LSP.Protocol.Lens                   as L
 import           Language.LSP.Protocol.Message
 import           Language.LSP.Protocol.Types
 import           Language.LSP.Server
-import           Language.LSP.VFS                             (virtualFileText)
+#if MIN_VERSION_ghc(9,11,0)
+import           GHC.Unit.Module.ModIface                     (IfaceTopEnv (..))
+#endif
+
 
 {- | Code Lens provider
  NOTE: Invoked every time the document is modified, not just when the document is saved.
@@ -151,14 +142,6 @@ codeLens recorder st plId CodeLensParams{_textDocument} =
                 dbg $ LogCodeLensFp fp
                 (comments, _) <-
                     runActionE "eval.GetParsedModuleWithComments" st $ useWithStaleE GetEvalComments nfp
-                -- dbg "excluded comments" $ show $  DL.toList $
-                --     foldMap (\(L a b) ->
-                --         case b of
-                --             AnnLineComment{}  -> mempty
-                --             AnnBlockComment{} -> mempty
-                --             _                 -> DL.singleton (a, b)
-                --     )
-                --     $ apiAnnComments' pm_annotations
                 dbg $ LogCodeLensComments comments
 
                 -- Extract tests from source code
@@ -210,7 +193,7 @@ runEvalCmd recorder plId st mtoken EvalParams{..} =
             let TextDocumentIdentifier{_uri} = module_
             fp <- uriToFilePathE _uri
             let nfp = toNormalizedFilePath' fp
-            mdlText <- moduleText _uri
+            mdlText <- moduleText st _uri
 
             -- enable codegen for the module which we need to evaluate.
             final_hscEnv <- liftIO $ bracket_
@@ -260,17 +243,22 @@ initialiseSessionForEval needs_quickcheck st nfp = do
     -- However, the eval plugin (setContext specifically) requires the rdr_env
     -- for the current module - so get it from the Typechecked Module and add
     -- it back to the iface for the current module.
-    rdr_env <- tcg_rdr_env . tmrTypechecked <$> use_ TypeCheck nfp
+    tm <- tmrTypechecked <$> use_ TypeCheck nfp
+    let rdr_env = tcg_rdr_env tm
     let linkable_hsc = loadModulesHome (map (addRdrEnv . linkableHomeMod) linkables) deps_hsc
         addRdrEnv hmi
           | iface <- hm_iface hmi
           , ms_mod ms == mi_module iface
+#if MIN_VERSION_ghc(9,11,0)
+          = hmi { hm_iface = set_mi_top_env (Just $ IfaceTopEnv (forceGlobalRdrEnv (globalRdrEnvLocal rdr_env)) (mkIfaceImports $ tcg_import_decls tm)) iface}
+#else
           = hmi { hm_iface = iface { mi_globals = Just $!
 #if MIN_VERSION_ghc(9,8,0)
                     forceGlobalRdrEnv
 #endif
                       rdr_env
                 }}
+#endif
           | otherwise = hmi
 
     return (ms, linkable_hsc)
@@ -291,6 +279,15 @@ initialiseSessionForEval needs_quickcheck st nfp = do
             getSession
   return env2
 
+#if MIN_VERSION_ghc(9,11,0)
+mkIfaceImports :: [ImportUserSpec] -> [IfaceImport]
+mkIfaceImports = map go
+  where
+    go (ImpUserSpec decl ImpUserAll) = IfaceImport decl ImpIfaceAll
+    go (ImpUserSpec decl (ImpUserExplicit env)) = IfaceImport decl (ImpIfaceExplicit (forceGlobalRdrEnv env))
+    go (ImpUserSpec decl (ImpUserEverythingBut ns)) = IfaceImport decl (ImpIfaceEverythingBut ns)
+#endif
+
 addFinalReturn :: Text -> [TextEdit] -> [TextEdit]
 addFinalReturn mdlText edits
     | not (null edits) && not (T.null mdlText) && T.last mdlText /= '\n' =
@@ -305,12 +302,15 @@ finalReturn txt =
         p = Position l c
      in TextEdit (Range p p) "\n"
 
-moduleText :: Uri -> ExceptT PluginError (HandlerM config) Text
-moduleText uri =
-    handleMaybeM (PluginInternalError "mdlText") $
-      (virtualFileText <$>)
-          <$> pluginGetVirtualFile
-              (toNormalizedUri uri)
+moduleText :: IdeState -> Uri -> ExceptT PluginError (HandlerM config) Text
+moduleText state uri = do
+    contents <-
+        handleMaybeM (PluginInternalError "mdlText") $
+            liftIO $
+                runAction "eval.getUriContents" state $
+                    getUriContents $
+                        toNormalizedUri uri
+    pure $ Rope.toText contents
 
 testsBySection :: [Section] -> [(Section, EvalId, Test)]
 testsBySection sections =
@@ -514,7 +514,7 @@ singleLine s = [T.pack s]
 errorLines :: String -> [Text]
 errorLines =
         dropWhileEnd T.null
-        . takeWhile (not . ("CallStack" `T.isPrefixOf`))
+        . takeWhile (not . (\x -> "CallStack" `T.isPrefixOf` x || "HasCallStack" `T.isPrefixOf` x))
         . T.lines
         . T.pack
 
@@ -654,7 +654,6 @@ data GhciLikeCmdException = GhciLikeCmdNotImplemented
     { ghciCmdName :: Text
     , ghciCmdArg  :: Text
     }
-    deriving (Typeable)
 
 instance Show GhciLikeCmdException where
     showsPrec _ GhciLikeCmdNotImplemented{..} =
