@@ -6,15 +6,17 @@
 {-# LANGUAGE RecordWildCards    #-}
 {-# LANGUAGE TypeFamilies       #-}
 {-# LANGUAGE ViewPatterns       #-}
+
 module Ide.Plugin.ExplicitImports
   ( descriptor
   , descriptorForModules
   , abbreviateImportTitle
+  , abbreviateImportTitleWithoutModule
   , Log(..)
   ) where
 
 import           Control.DeepSeq
-import           Control.Lens                         ((&), (?~))
+import           Control.Lens                         (_Just, (&), (?~), (^?))
 import           Control.Monad.Error.Class            (MonadError (throwError))
 import           Control.Monad.IO.Class
 import           Control.Monad.Trans.Class            (lift)
@@ -22,14 +24,19 @@ import           Control.Monad.Trans.Except           (ExceptT)
 import           Control.Monad.Trans.Maybe
 import qualified Data.Aeson                           as A (ToJSON (toJSON))
 import           Data.Aeson.Types                     (FromJSON)
+import           Data.Char                            (isSpace)
+import           Data.Functor                         ((<&>))
 import qualified Data.IntMap                          as IM (IntMap, elems,
                                                              fromList, (!?))
 import           Data.IORef                           (readIORef)
+import           Data.List                            (singleton)
 import qualified Data.Map.Strict                      as Map
-import           Data.Maybe                           (isNothing, mapMaybe)
+import           Data.Maybe                           (isJust, isNothing,
+                                                       mapMaybe)
 import qualified Data.Set                             as S
 import           Data.String                          (fromString)
 import qualified Data.Text                            as T
+import qualified Data.Text                            as Text
 import           Data.Traversable                     (for)
 import qualified Data.Unique                          as U (hashUnique,
                                                             newUnique)
@@ -44,11 +51,14 @@ import           GHC.Generics                         (Generic)
 import           Ide.Plugin.Error                     (PluginError (..),
                                                        getNormalizedFilePathE,
                                                        handleMaybe)
-import           Ide.Plugin.RangeMap                  (filterByRange)
-import qualified Ide.Plugin.RangeMap                  as RM (RangeMap, fromList)
+import qualified Ide.Plugin.RangeMap                  as RM (RangeMap,
+                                                             filterByRange,
+                                                             fromList)
 import           Ide.Plugin.Resolve
 import           Ide.PluginUtils
 import           Ide.Types
+import           Language.LSP.Protocol.Lens           (HasInlayHint (inlayHint),
+                                                       HasTextDocument (textDocument))
 import qualified Language.LSP.Protocol.Lens           as L
 import           Language.LSP.Protocol.Message
 import           Language.LSP.Protocol.Types
@@ -76,7 +86,7 @@ instance Pretty Log where
 descriptor :: Recorder (WithPriority Log) -> PluginId -> PluginDescriptor IdeState
 descriptor recorder =
     -- (almost) no one wants to see an explicit import list for Prelude
-    descriptorForModules recorder (/= moduleName pRELUDE)
+    descriptorForModules recorder (/= pRELUDE_NAME)
 
 descriptorForModules
     :: Recorder (WithPriority Log)
@@ -97,17 +107,23 @@ descriptorForModules recorder modFilter plId =
          -- This plugin provides code lenses
            mkPluginHandler SMethod_TextDocumentCodeLens (lensProvider recorder)
         <> mkResolveHandler SMethod_CodeLensResolve (lensResolveProvider recorder)
-          -- This plugin provides code actions
+        -- This plugin provides inlay hints
+        <> mkPluginHandler SMethod_TextDocumentInlayHint (inlayHintProvider recorder)
+        -- This plugin provides code actions
         <> codeActionHandlers
-
     }
 
+isInlayHintsSupported :: IdeState -> Bool
+isInlayHintsSupported ideState =
+  let clientCaps = Shake.clientCapabilities $ shakeExtras ideState
+   in isJust $ clientCaps ^? textDocument . _Just . inlayHint . _Just
+
 -- | The actual command handler
 runImportCommand :: Recorder (WithPriority Log) -> CommandFunction IdeState IAResolveData
 runImportCommand recorder ideState _ eird@(ResolveOne _ _) = do
   wedit <- resolveWTextEdit ideState eird
   _ <- lift $ pluginSendRequest SMethod_WorkspaceApplyEdit (ApplyWorkspaceEditParams Nothing wedit) logErrors
-  return $ InR  Null
+  return $ InR Null
   where logErrors (Left re) = do
           logWith recorder Error (LogWAEResponseError re)
           pure ()
@@ -129,12 +145,18 @@ runImportCommand _ _ _ rd = do
 -- the provider should produce one code lens associated to the import statement:
 -- > Refine imports to import Control.Monad.IO.Class (liftIO)
 lensProvider :: Recorder (WithPriority Log) -> PluginMethodHandler IdeState 'Method_TextDocumentCodeLens
-lensProvider _  state _ CodeLensParams {_textDocument = TextDocumentIdentifier {_uri}} = do
+lensProvider _ state _ CodeLensParams {_textDocument = TextDocumentIdentifier {_uri}} = do
     nfp <- getNormalizedFilePathE _uri
     (ImportActionsResult{forLens}, pm) <- runActionE "ImportActions" state $ useWithStaleE ImportActions nfp
     let lens = [ generateLens _uri newRange int
-                | (range, int) <- forLens
-                , Just newRange <- [toCurrentRange pm range]]
+               -- provide ExplicitImport only if the client does not support inlay hints
+               | not (isInlayHintsSupported state)
+               , (range, (int, ExplicitImport)) <- forLens
+               , Just newRange <- [toCurrentRange pm range]] <>
+               -- RefineImport is always provided because inlay hints cannot
+               [ generateLens _uri newRange int
+               | (range, (int, RefineImport)) <- forLens
+               , Just newRange <- [toCurrentRange pm range]]
     pure $ InL lens
   where -- because these are non resolved lenses we only need the range and a
         -- unique id to later resolve them with. These are for both refine
@@ -145,12 +167,13 @@ lensProvider _  state _ CodeLensParams {_textDocument = TextDocumentIdentifier {
                    , _range = range
                    , _command = Nothing }
 
+
 lensResolveProvider :: Recorder (WithPriority Log) -> ResolveFunction IdeState IAResolveData 'Method_CodeLensResolve
 lensResolveProvider _ ideState plId cl uri rd@(ResolveOne _ uid) = do
     nfp <- getNormalizedFilePathE uri
     (ImportActionsResult{forResolve}, _) <- runActionE "ImportActions" ideState $ useWithStaleE ImportActions nfp
     target <- handleMaybe PluginStaleResolve $ forResolve IM.!? uid
-    let updatedCodeLens = cl & L.command ?~  mkCommand plId target
+    let updatedCodeLens = cl & L.command ?~ mkCommand plId target
     pure updatedCodeLens
   where mkCommand ::  PluginId -> ImportEdit -> Command
         mkCommand pId (ImportEdit{ieResType, ieText}) =
@@ -165,6 +188,53 @@ lensResolveProvider _ ideState plId cl uri rd@(ResolveOne _ uid) = do
 lensResolveProvider _ _ _ _ _ rd = do
    throwError $ PluginInvalidParams (T.pack $ "Unexpected argument for lens resolve handler: " <> show rd)
 
+
+-- | Provide explicit imports in inlay hints.
+-- Applying textEdits can make the import explicit.
+-- There is currently no need to resolve inlay hints,
+-- as no tooltips or commands are provided in the label.
+inlayHintProvider :: Recorder (WithPriority Log) -> PluginMethodHandler IdeState 'Method_TextDocumentInlayHint
+inlayHintProvider _ state _ InlayHintParams {_textDocument = TextDocumentIdentifier {_uri}, _range = visibleRange} =
+    if isInlayHintsSupported state
+    then do
+        nfp <- getNormalizedFilePathE _uri
+        (ImportActionsResult {forLens, forResolve}, pm) <- runActionE "ImportActions" state $ useWithStaleE ImportActions nfp
+        let inlayHints = [ inlayHint
+                         | (range, (int, _)) <- forLens
+                         , Just newRange <- [toCurrentRange pm range]
+                         , isSubrangeOf newRange visibleRange
+                         , Just ie <- [forResolve IM.!? int]
+                         , Just inlayHint <- [generateInlayHints newRange ie pm]]
+        pure $ InL inlayHints
+    -- When the client does not support inlay hints, fallback to the code lens,
+    -- so there is nothing to response here.
+    -- `[]` is no different from `null`, we chose to use all `[]` to indicate "no information"
+    else pure $ InL []
+  where
+    -- The appropriate and intended position for the hint hints to begin
+    -- is the end of the range for the code lens.
+    --   import Data.Char (isSpace)
+    --   |--- range ----|-- IH ---|
+    --                  |^-_paddingLeft
+    --                  ^-_position
+    generateInlayHints :: Range -> ImportEdit -> PositionMapping -> Maybe InlayHint
+    generateInlayHints (Range _ end) ie pm = mkLabel ie <&> \label ->
+      InlayHint { _position = end
+                , _label = InL label
+                , _kind = Nothing -- neither a type nor a parameter
+                , _textEdits = fmap singleton $ toTEdit pm ie
+                , _tooltip = Just $ InL "Make this import explicit" -- simple enough, no need to resolve
+                , _paddingLeft = Just True -- show an extra space before the inlay hint
+                , _paddingRight = Nothing
+                , _data_ = Nothing
+                }
+    mkLabel :: ImportEdit -> Maybe T.Text
+    mkLabel (ImportEdit{ieResType, ieText}) =
+      let title ExplicitImport = Just $ abbreviateImportTitleWithoutModule ieText
+          title RefineImport   = Nothing -- does not provide imports statements that can be refined via inlay hints
+      in title ieResType
+
+
 -- |For explicit imports: If there are any implicit imports, provide both one
 -- code action per import to make that specific import explicit, and one code
 -- action to turn them all into explicit imports. For refine imports: If there
@@ -175,7 +245,7 @@ codeActionProvider _ ideState _pId (CodeActionParams _ _ TextDocumentIdentifier
     nfp <- getNormalizedFilePathE _uri
     (ImportActionsResult{forCodeActions}, pm) <- runActionE "ImportActions" ideState $ useWithStaleE ImportActions nfp
     newRange <- toCurrentRangeE pm range
-    let relevantCodeActions = filterByRange newRange forCodeActions
+    let relevantCodeActions = RM.filterByRange newRange forCodeActions
         allExplicit =
           [InR $ mkCodeAction "Make all imports explicit" (Just $ A.toJSON $ ExplicitAll _uri)
           -- We should only provide this code action if there are any code
@@ -231,12 +301,14 @@ resolveWTextEdit ideState (RefineAll uri) = do
   pure $ mkWorkspaceEdit uri edits pm
 mkWorkspaceEdit :: Uri -> [ImportEdit] -> PositionMapping -> WorkspaceEdit
 mkWorkspaceEdit uri edits pm =
-      WorkspaceEdit {_changes = Just $ Map.singleton uri  (mapMaybe toWEdit edits)
+      WorkspaceEdit {_changes = Just $ Map.singleton uri (mapMaybe (toTEdit pm) edits)
                     , _documentChanges = Nothing
                     , _changeAnnotations = Nothing}
-  where toWEdit ImportEdit{ieRange, ieText} =
-          let newRange = toCurrentRange pm ieRange
-          in (\r -> TextEdit r ieText) <$> newRange
+
+toTEdit :: PositionMapping -> ImportEdit -> Maybe TextEdit
+toTEdit pm ImportEdit{ieRange, ieText} =
+      let newRange = toCurrentRange pm ieRange
+      in (\r -> TextEdit r ieText) <$> newRange
 
 data ImportActions = ImportActions
   deriving (Show, Generic, Eq, Ord)
@@ -254,7 +326,7 @@ data ImportActionsResult = ImportActionsResult
   { -- |For providing the code lenses we need to have a range, and a unique id
     -- that is later resolved to the new text for each import. It is stored in
     -- a list, because we always need to provide all the code lens in a file.
-    forLens        :: [(Range, Int)]
+    forLens        :: [(Range, (Int, ResultType))]
     -- |For the code actions we have the same data as for the code lenses, but
     -- we store it in a RangeMap, because that allows us to filter on a specific
     -- range with better performance, and code actions are almost always only
@@ -331,7 +403,7 @@ minimalImportsRule recorder modFilter = defineNoDiagnostics (cmapWithPrio LogSha
         -- for every minimal imports
         | (location, origImport, minImport@(ImportDecl{ideclName = L _ mn})) <- locationImportWithMinimal
         -- (almost) no one wants to see an refine import list for Prelude
-        , mn /= moduleName pRELUDE
+        , mn /= pRELUDE_NAME
         -- we check for the inner imports
         , Just innerImports <- [Map.lookup mn import2Map]
         -- and only get those symbols used
@@ -346,7 +418,7 @@ minimalImportsRule recorder modFilter = defineNoDiagnostics (cmapWithPrio LogSha
                                 pure (u,  rt)
   let rangeAndUnique =  [ ImportAction r u rt | (u, (r, (_, rt))) <- uniqueAndRangeAndText ]
   pure ImportActionsResult
-                      { forLens = (\ImportAction{..} -> (iaRange, iaUniqueId)) <$> rangeAndUnique
+                      { forLens = (\ImportAction{..} -> (iaRange, (iaUniqueId, iaResType))) <$> rangeAndUnique
                       , forCodeActions = RM.fromList iaRange rangeAndUnique
                       , forResolve =  IM.fromList ((\(u, (r, (te, ty))) -> (u, ImportEdit r te ty)) <$> uniqueAndRangeAndText) }
 
@@ -413,8 +485,6 @@ isExplicitImport _                                                = False
 maxColumns :: Int
 maxColumns = 120
 
-
--- | The title of the command is ideally the minimal explicit import decl, but
 -- we don't want to create a really massive code lens (and the decl can be extremely large!).
 -- So we abbreviate it to fit a max column size, and indicate how many more items are in the list
 -- after the abbreviation
@@ -422,7 +492,8 @@ abbreviateImportTitle :: T.Text -> T.Text
 abbreviateImportTitle input =
   let
       -- For starters, we only want one line in the title
-      oneLineText = T.unwords $ T.lines input
+      -- we also need to compress multiple spaces into one
+      oneLineText = T.unwords $ filter (not . T.null) $ T.split isSpace input
       -- Now, split at the max columns, leaving space for the summary text we're going to add
       -- (conservatively assuming we won't need to print a number larger than 100)
       (prefix, suffix) = T.splitAt (maxColumns - T.length (summaryText 100)) oneLineText
@@ -447,6 +518,11 @@ abbreviateImportTitle input =
           else actualPrefix <> suffixText
   in title
 
+-- Create an import abbreviate title without module for inlay hints
+abbreviateImportTitleWithoutModule :: Text.Text -> Text.Text
+abbreviateImportTitleWithoutModule = abbreviateImportTitle . T.dropWhile (/= '(')
+
+-- | The title of the command is ideally the minimal explicit import decl, but
 --------------------------------------------------------------------------------
 
 
@@ -464,11 +540,7 @@ filterByImport (ImportDecl{ideclHiding = Just (_, L _ names)})
     then Just res
     else Nothing
   where importedNames = S.fromList $ map (ieName . unLoc) names
-        res = flip Map.filter avails $ \a ->
-                any (`S.member` importedNames)
-                  $ concatMap
-                      getAvailNames
-                      a
+        res = Map.filter (any (any (`S.member` importedNames) . getAvailNames)) avails
         allFilteredAvailsNames = S.fromList
           $ concatMap getAvailNames
           $ mconcat
