@@ -17,8 +17,9 @@ module Ide.Plugin.Pragmas
 
 import           Control.Lens                             hiding (List)
 import           Control.Monad.IO.Class                   (MonadIO (liftIO))
-import           Control.Monad.Trans.Class                (lift)
+import qualified Data.Aeson                               as JSON
 import           Data.Char                                (isAlphaNum)
+import qualified Data.Foldable                            as Foldable
 import           Data.List.Extra                          (nubOrdOn)
 import qualified Data.Map                                 as M
 import           Data.Maybe                               (mapMaybe)
@@ -29,7 +30,7 @@ import           Development.IDE.Core.Compile             (sourceParser,
 import           Development.IDE.Core.PluginUtils
 import           Development.IDE.GHC.Compat
 import           Development.IDE.Plugin.Completions       (ghcideCompletionsPluginPriority)
-import           Development.IDE.Plugin.Completions.Logic (getCompletionPrefix)
+import           Development.IDE.Plugin.Completions.Logic (getCompletionPrefixFromRope)
 import           Development.IDE.Plugin.Completions.Types (PosPrefixInfo (..))
 import qualified Development.IDE.Spans.Pragmas            as Pragmas
 import           Ide.Plugin.Error
@@ -80,7 +81,7 @@ mkCodeActionProvider mkSuggest state _plId
     -- ghc session to get some dynflags even if module isn't parsed
     (hscEnv -> hsc_dflags -> sessionDynFlags, _) <-
       runActionE "Pragmas.GhcSession" state $ useWithStaleE GhcSession normalizedFilePath
-    (_, fileContents) <- liftIO $ runAction "Pragmas.GetFileContents" state $ getFileContents normalizedFilePath
+    fileContents <- liftIO $ runAction "Pragmas.GetFileContents" state $ getFileContents normalizedFilePath
     parsedModule <- liftIO $ runAction "Pragmas.GetParsedModule" state $ getParsedModule normalizedFilePath
     let parsedModuleDynFlags = ms_hspp_opts . pm_mod_summary <$> parsedModule
         nextPragmaInfo = Pragmas.getNextPragmaInfo sessionDynFlags fileContents
@@ -121,15 +122,23 @@ suggest dflags diag =
 -- ---------------------------------------------------------------------
 
 suggestDisableWarning :: Diagnostic -> [PragmaEdit]
-suggestDisableWarning Diagnostic {_code}
-  | Just (LSP.InR (T.stripPrefix "-W" -> Just w)) <- _code
-  , w `notElem` warningBlacklist =
-    pure ("Disable \"" <> w <> "\" warnings", OptGHC w)
+suggestDisableWarning diagnostic
+  | Just (Just (JSON.Array attachedReasons)) <- diagnostic ^? attachedReason
+    =
+    [ ("Disable \"" <> w <> "\" warnings", OptGHC w)
+    | JSON.String attachedReason <- Foldable.toList attachedReasons
+    , Just w <- [T.stripPrefix "-W" attachedReason]
+    , w `notElem` warningBlacklist
+    ]
   | otherwise = []
 
--- Don't suggest disabling type errors as a solution to all type errors
 warningBlacklist :: [T.Text]
-warningBlacklist = ["deferred-type-errors"]
+warningBlacklist =
+  -- Don't suggest disabling type errors as a solution to all type errors.
+  [ "deferred-type-errors"
+  -- Don't suggest disabling out of scope errors as a solution to all out of scope errors.
+  , "deferred-out-of-scope-variables"
+  ]
 
 -- ---------------------------------------------------------------------
 
@@ -195,13 +204,13 @@ flags :: [T.Text]
 flags = map T.pack $ flagsForCompletion False
 
 completion :: PluginMethodHandler IdeState 'LSP.Method_TextDocumentCompletion
-completion _ide _ complParams = do
+completion ide _ complParams = do
     let (LSP.TextDocumentIdentifier uri) = complParams ^. L.textDocument
         position@(Position ln col) = complParams ^. L.position
-    contents <- lift $ pluginGetVirtualFile $ toNormalizedUri uri
+    contents <- liftIO $ runAction "Pragmas.GetUriContents" ide $ getUriContents $ toNormalizedUri uri
     fmap LSP.InL $ case (contents, uriToFilePath' uri) of
         (Just cnts, Just _path) ->
-            pure $ result $ getCompletionPrefix position cnts
+            pure $ result $ getCompletionPrefixFromRope position cnts
             where
                 result pfix
                     | "{-# language" `T.isPrefixOf` line
