@@ -42,6 +42,7 @@ import           Data.Monoid                          (First (First))
 import           Data.String                          (IsString)
 import qualified Data.Text                            as T
 import qualified Data.Text.Encoding                   as T
+import qualified Data.Text.Utf16.Rope.Mixed           as Rope
 import           Development.IDE                      hiding (pluginHandlers)
 import           Development.IDE.Core.Actions         (lookupMod)
 import           Development.IDE.Core.PluginUtils
@@ -127,9 +128,7 @@ import           Retrie.SYB                           (everything, extQ,
 import           Retrie.Types
 import           Retrie.Universe                      (Universe)
 
-#if MIN_VERSION_ghc(9,3,0)
 import           GHC.Types.PkgQual
-#endif
 
 data Log
   = LogParsingModule FilePath
@@ -503,7 +502,7 @@ data CallRetrieError
   | NoParse NormalizedFilePath
   | GHCParseError NormalizedFilePath String
   | NoTypeCheck NormalizedFilePath
-  deriving (Eq, Typeable)
+  deriving (Eq)
 
 instance Show CallRetrieError where
   show (CallRetrieInternalError msg f) = msg <> " - " <> fromNormalizedFilePath f
@@ -735,11 +734,7 @@ toImportDecl AddImport {..} = GHC.ImportDecl {ideclSource = ideclSource', ..}
     ideclAs = toMod <$> ideclAsString
     ideclQualified = if ideclQualifiedBool then GHC.QualifiedPre else GHC.NotQualified
 
-#if MIN_VERSION_ghc(9,3,0)
     ideclPkgQual = NoRawPkgQual
-#else
-    ideclPkgQual = Nothing
-#endif
 
 #if MIN_VERSION_ghc(9,5,0)
     ideclImportList = Nothing
@@ -784,10 +779,10 @@ getCPPmodule recorder state session t = do
           return (fixities, parsed)
 
     contents <- do
-      (_, mbContentsVFS) <-
+      mbContentsVFS <-
         runAction "Retrie.GetFileContents" state $ getFileContents nt
       case mbContentsVFS of
-        Just contents -> return contents
+        Just contents -> return $ Rope.toText contents
         Nothing       -> T.decodeUtf8 <$> BS.readFile (fromNormalizedFilePath nt)
     if any (T.isPrefixOf "#if" . T.toLower) (T.lines contents)
       then do
